@@ -1,5 +1,5 @@
 #  Phusion Passenger - https://www.phusionpassenger.com/
-#  Copyright (c) 2010-2015 Phusion Holding B.V.
+#  Copyright (c) 2010-2017 Phusion Holding B.V.
 #
 #  "Passenger", "Phusion Passenger" and "Union Station" are registered
 #  trademarks of Phusion Holding B.V.
@@ -24,6 +24,7 @@
 
 require 'optparse'
 require 'thread'
+require 'socket'
 PhusionPassenger.require_passenger_lib 'constants'
 PhusionPassenger.require_passenger_lib 'ruby_core_enhancements'
 PhusionPassenger.require_passenger_lib 'standalone/command'
@@ -66,7 +67,7 @@ module PhusionPassenger
           touch_temp_dir_in_background
           ########################
           ########################
-          watch_log_files_in_background if should_watch_logs?
+          watch_log_files_in_background if should_watch_one_or_more_log_files?
           wait_until_engine_has_exited if should_wait_until_engine_has_exited?
         rescue Interrupt
           trapsafe_shutdown_and_cleanup(true)
@@ -97,7 +98,7 @@ module PhusionPassenger
         # If you add or change an option, make sure to update the following places too:
         # - src/ruby_supportlib/phusion_passenger/standalone/start_command/builtin_engine.rb,
         #   function #build_daemon_controller_options
-        # - resources/templates/config/standalone.erb
+        # - resources/templates/standalone/config.erb
         OptionParser.new do |opts|
           defaults = CONFIG_DEFAULTS
           nl = "\n" + ' ' * 37
@@ -124,11 +125,6 @@ module PhusionPassenger
           ConfigUtils.add_option_parser_options_from_config_spec(opts,
             REQUEST_HANDLING_CONFIG_SPECS, options)
 
-          opts.separator ""
-          opts.separator "Union Station options:"
-          ConfigUtils.add_option_parser_options_from_config_spec(opts,
-            UNION_STATION_CONFIG_SPECS, options)
-
           opts.separator ""
           opts.separator "Nginx engine options:"
           ConfigUtils.add_option_parser_options_from_config_spec(opts,
@@ -176,6 +172,9 @@ module PhusionPassenger
         if @options[:ssl] && !@options[:ssl_certificate_key]
           abort "You specified --ssl. Please specify --ssl-certificate-key as well."
         end
+        if @options[:nginx_tarball] && !@options_without_defaults[:nginx_version]
+          abort "You specified --nginx-tarball. Please also specify which Nginx version the tarball contains using --nginx-version."
+        end
         if @options[:engine] != "builtin" && @options[:engine] != "nginx"
           abort "You've specified an invalid value for --engine. The only values allowed are: builtin, nginx."
         end
@@ -217,10 +216,10 @@ module PhusionPassenger
         else
           nginx_name = "nginx-#{@options[:nginx_version]}"
           @nginx_binary = PhusionPassenger.find_support_binary(nginx_name)
-          if !@agent_exe || !@nginx_binary
+          if !@agent_exe || (@options[:engine] == "nginx" && !@nginx_binary)
             install_runtime
             @agent_exe = PhusionPassenger.find_support_binary(AGENT_EXE)
-            @nginx_binary = PhusionPassenger.find_support_binary(nginx_name)
+            @nginx_binary = PhusionPassenger.find_support_binary(nginx_name) if @options[:engine] == "nginx"
           end
         end
       end
@@ -239,6 +238,7 @@ module PhusionPassenger
           # (as opposed to responding quickly with an error), then the system
           # quickly switches to a mirror.
           "--connect-timeout", "0",
+          "--engine", @options[:engine],
           "--idle-timeout", "0"
         ]
         if @options[:auto]
@@ -248,13 +248,15 @@ module PhusionPassenger
           args << "--url-root"
           args << @options[:binaries_url_root]
         end
-        if @options[:nginx_version]
-          args << "--nginx-version"
-          args << @options[:nginx_version]
-        end
-        if @options[:nginx_tarball]
-          args << "--nginx-tarball"
-          args << @options[:nginx_tarball]
+        if @options[:engine] == "nginx"
+          if @options[:nginx_version]
+            args << "--nginx-version"
+            args << @options[:nginx_version]
+          end
+          if @options[:nginx_tarball]
+            args << "--nginx-tarball"
+            args << @options[:nginx_tarball]
+          end
         end
         if @options[:dont_compile_runtime]
           args << "--no-compile"
@@ -279,7 +281,7 @@ module PhusionPassenger
 
       def find_apps
         PhusionPassenger.require_passenger_lib 'standalone/app_finder'
-        @app_finder = AppFinder.new(@argv, @options)
+        @app_finder = AppFinder.new(@argv, @options, @local_options)
         @apps = @app_finder.scan
         if @app_finder.multi_mode? && @options[:engine] != 'nginx'
           puts "Mass deployment enabled, so forcing engine to 'nginx'."
@@ -400,13 +402,16 @@ module PhusionPassenger
       end
 
       def touch_temp_dir_in_background
-        result = system(@agent_exe,
+        command = [@agent_exe,
           "temp-dir-toucher",
           @working_dir,
           "--cleanup",
           "--daemonize",
           "--pid-file", "#{@working_dir}/temp_dir_toucher.pid",
-          "--log-file", @options[:log_file])
+          "--log-file", @options[:log_file]]
+        command << "--user" << @options[:user] unless @options[:user].nil?
+        command << "--nginx-pid" << @engine.pid.to_s if @options[:engine] == 'nginx'
+        result = system(*command)
         if !result
           abort "Cannot start #{@agent_exe} temp-dir-toucher"
         end
@@ -455,15 +460,34 @@ module PhusionPassenger
           @threads << thread
           @interruptable_threads << thread
         end
-        thread = Utils.create_thread_and_abort_on_exception do
-          watch_log_file(@options[:log_file])
+        if should_watch_main_log_file?
+          thread = Utils.create_thread_and_abort_on_exception do
+            watch_log_file(@options[:log_file])
+          end
+          @threads << thread
+          @interruptable_threads << thread
         end
-        @threads << thread
-        @interruptable_threads << thread
       end
 
-      def should_watch_logs?
-        return !@options[:daemonize] && @options[:log_file] != "/dev/null"
+      def should_watch_one_or_more_log_files?
+        !@options[:daemonize]
+      end
+
+      def should_watch_main_log_file?
+        if @options[:daemonize]
+          false
+        else
+          begin
+            stat = File.stat(@options[:log_file])
+          rescue Errno::ENOENT
+            stat = nil
+          end
+          if stat
+            stat.file?
+          else
+            true
+          end
+        end
       end
 
       def should_wait_until_engine_has_exited?
@@ -507,6 +531,12 @@ module PhusionPassenger
             @engine.stop
             STDOUT.puts " done"
             STDOUT.flush
+            if @options[:engine] == "nginx" && @options[:socket_file]
+              begin
+                File.delete(@options[:socket_file])
+              rescue Errno::ENOENT
+              end
+            end
           end
           @engine = nil
         end
